#include "mex.h"
#include <math.h>
#include <string.h>

// To be used by
//   [f, g] = group_PO_smooth_obj(y, s, sigma, graph)
// Input:
//   y: current solution vector (dimension = #group)
//   s: weight vector with dimension #group
//   sigma: the c in the paper
//   graph, sized #group * #variable, is a sparse indicator
// Output:
//   f: the objective value of the smooth objective
//   g: the gradient of the smooth objective
void mexFunction (int nlhs, mxArray *plhs[], 
                  int nrhs, const mxArray *prhs[]) 
{
  mwIndex *jc, *ir, nnz, i, j;
  double sigma, tmp, *s, *y, *g, f, maxy, *buf, sumexp;
  size_t num_group = mxGetM(prhs[3]);
  size_t num_var = mxGetN(prhs[3]);

  y = mxGetPr(prhs[0]);
  s = mxGetPr(prhs[1]);
  sigma = *mxGetPr(prhs[2]);
  jc = mxGetJc(prhs[3]);
  ir = mxGetIr(prhs[3]);

  plhs[1] = mxCreateDoubleMatrix(num_group, 1, mxREAL);

  g = mxGetPr(plhs[1]);
  buf = new double [num_group];

  memset(g, 0, num_group*sizeof(double));

  f = 0.0;
  for (i = 0; i < num_var; i ++)
  {
    maxy = -2;
    tmp = s[i]/sigma;
    nnz = jc[i+1] - jc[i];  
    // for all groups involving node i
    for (j=0; j < nnz; j++) 
    {
      buf[j] = -y[*(ir+j)];
      if (buf[j] > maxy)
        maxy = buf[j];
    }
    sumexp = 0.0;
    for (j=0; j < nnz; j++)
    {
      buf[j] = exp((buf[j] - maxy)*tmp);
      sumexp += buf[j];
    }
    f += log(sumexp) + maxy*tmp;

    // set the gradient
    tmp = -s[i]/sumexp;
    for (j = 0; j < nnz; j++)
      g[*(ir++)] += buf[j]*tmp;
  }
  
  plhs[0] = mxCreateDoubleScalar(f*sigma);

  delete [] buf;
}
