#include "mex.h"
#include <math.h>

// To be used by
//  [reg] = reg_group_inf(x, graph)
//  It works ONLY for L_\infty norm (hence Lovasz extension is sum of L_\infty)
// Input: 
//   x: the value vector on the nodes
//   graph: the sparse matrix (#var * #group) that encodes group structure
// Output:
//   reg: the value of group regularizer
void mexFunction (int nlhs, mxArray *plhs[], 
                  int nrhs, const mxArray *prhs[]) 
{
  mwIndex num_group;
  mwIndex *jc, *ir;
  mwIndex i, j, nnz;  
  double *x, reg, tmp, group_max;

  x = mxGetPr(prhs[0]);
  jc = mxGetJc(prhs[1]);    ir = mxGetIr(prhs[1]);

  num_group = mxGetN(prhs[1]);

  reg = 0.0;
  for (i = 0; i < num_group; i++)  
  {    
    nnz = jc[i+1] - jc[i];
    
    group_max = -1;
    for (j = 0; j < nnz; j ++)
    {
      tmp = fabs(x[*(ir++)]);
      if (tmp > group_max)
        group_max = tmp;      
    }
    reg += group_max;
  }

  plhs[0] = mxCreateDoubleScalar(reg);
}
