function [u, v] = get_basis(Xk, opts)
  
  [G1, G2, grads] = comp_grad_mv(Xk, opts);  

  opts.cp_max_iter = 40;
  opts.cp_eig_tol = 1e-5;
  d1 = opts.num_f1;
  d2 = opts.num_f2;
  ub = opts.ub;
  
  % prepare context for solving SD
  ctx = [];
  ctx.precomp = true;
  ctx.A = G1' * (opts.weight_X1 * opts.ub(1));
  ctx.B = G2' * (opts.weight_X2 * opts.ub(2));
  ctx.AtA = ctx.A'*ctx.A;
  ctx.BtB = ctx.B'*ctx.B;
  ctx.AtB = ctx.A'*ctx.B;  

  obj_rec = -inf;
  grads = grads' * opts.weight_Y;
  sum_g = sum(grads, 2);
  base_estimate = sqrt(norm(ctx.A,'fro')^2+norm(ctx.B,'fro')^2);
  for i = 0:2^(opts.num_class-1)-1
    idx = [(dec2bin(i, opts.num_class-1) == '1'), false];
    ctx.c = 2*sum(grads(:,idx), 2) - sum_g;
    opts.estimate = (base_estimate + norm(ctx.c))^2;
    [obj, y, Z] = solve_SD(ctx, opts);
%     fprintf('i=%d, obj=%d\n', i, obj);
    if obj > obj_rec
      obj_rec = obj;  y_rec = y;  c_rec = ctx.c;  
      idx_rec = idx;  est_rec = opts.estimate;
    end
  end
  
  % recover Z
  ctA = c_rec'*ctx.A;
  ctB = c_rec'*ctx.B;
  Z = -[0, ctA, ctB
        ctA', ctx.AtA, ctx.AtB
        ctB', ctx.AtB', ctx.BtB];
  Z(1,1) = Z(1,1) - sum(y_rec);
  Z(2:d1+1, 2:d1+1) = Z(2:d1+1, 2:d1+1) + y_rec(2)*eye(d1);
  Z(d1+2:end, d1+2:end) = Z(d1+2:end, d1+2:end) + y_rec(3)*eye(d2);
  
  % Recover h
  
  h = recover_h(Z, d1, opts.ub);
  
  % do some local search to improve h

%   options = optimset('MaxIter', 20);  % 'GradConstr','on' , 'Display', 'off'
%   u = fmincon(@oracle_obj,h(2:end)*0.95,[],[],[],[],[],[],@ball_constraint,options);
  
  nest_opts = opts;
  nest_opts.init_L = sqrt(est_rec);
  nest_opts.max_iter = 10;
  nest_opts.verbose = 0;
  [u, obj, msg] = solve_Nesterov_generic(@oracle_obj, [], @prox_op, h(2:end), nest_opts);
  
  u = [u; 2*idx_rec'-1];
  v = ctx.A*u(1:d1) + ctx.B*u(d1+1:d1+d2) + c_rec;
  nv = norm(v);
  v = v / (-norm(v));
  qp = nv^2 - norm(c_rec)^2;
  fprintf('SD = %g, QP = %g, diff = %g, unorm=%g\n', obj_rec, qp, qp - obj_rec,...
          max(norm(u(1:d1)), norm(u(d1+1:d1+d2))));
 
  function [c,ceq,gradc,gradceq] = ball_constraint(x)
    ceq = []; 
    c(1) = norm(x(1:d1))^2 - opts.ub(1)^2;
    c(2) = norm(x(1+d1:end))^2 - opts.ub(2)^2;
    
    if nargout > 2
      gradceq = [];
      gradc = [2*x(1:d1), zeros(d1,1)
               zeros(d2,1), 2*x(1+d1:end)];
    end    
  end
    
  function [f,g] = oracle_obj(x)
    z = ctx.A*x(1:d1) + ctx.B*x(1+d1:end) + c_rec;
    f = -norm(z)^2;
    g = (-2)*[z'*ctx.A, z'*ctx.B]'; 
  end

  function uu = prox_op(uin, opts1)
    uu = uin;
    nm = norm(uin(1:d1)); if nm > ub(1), uu(1:d1) = uin(1:d1)*(ub(1)/nm);  end
    nm = norm(uin(1+d1:end)); if nm > ub(2), uu(1+d1:end) = uin(1+d1:end)*(ub(2)/nm);  end  
  end

end



