function [sol, obj, iter, msg, outputs] = ...
          multiview_alt_cons(lambda, X0, evaluator, opts)

% Solve  Min_X  func(X) + lambda * ||X||_*

m = size(X0, 1);
n = size(X0, 2);
k = opts.init_rank;  % Initialize U and V with rank k

U = randn(m, k);
V = randn(k, n);

make_output = (nargout == 5);

num_f1 = opts.num_f1;
num_f2 = opts.num_f2;
l = num_f1 + num_f2;
num_class = opts.num_class;
w1 = opts.weight_X1;
w2 = opts.weight_X2;
w3 = opts.weight_Y;
w = [w1, w2, w3];
ub1 = opts.ub(1);
ub2 = opts.ub(2);
assert(m == num_f1 + num_f2 + num_class);

if make_output
  outputs = [];
  outputs.obj_list = zeros(opts.max_iter, 1);
  outputs.time_list = zeros(opts.max_iter, 1);
  outputs.loss_list = zeros(opts.max_iter, 1);
  outputs.perf_list = zeros(opts.max_iter, 1);  
end

total_time = 0;     % Total time consumed by the algorithm
local_search_time = 0;  % Time spend on local search

global feval_logistic
global time_logistic

feval_logistic = 0;
time_logistic = 0;

perf = 0;
% Randomly initialize the bases
U1 = rand(num_f1, k);
U1 = U1 ./ repmat(sqrt(sum(U1.^2)*ub1), [num_f1, 1]);
U2 = rand(num_f2, k);
U2 = U2 ./ repmat(sqrt(sum(U2.^2)*ub2), [num_f2, 1]);
U3 = 2*(rand(num_class, k) - 0.5);
U = [U1; U2; U3];
V = rand(k, n);

pre_L_U = 1e-2;
pre_L_V = 1e-2;

for i = 1 : opts.max_iter
      
  t1 = cputime;  
  
  % Fix bases U, optimize the latent V
  n1 = sum(U(1:num_f1,:).^2);
  n2 = sum(U(1+num_f1:l,:).^2);
  nsq_list = U(1+l:end,:).^2;
  
  nest_opts = opts;
  nest_opts.init_L = pre_L_V;
  nest_opts.max_iter = 30;
  nest_opts.verbose = 0;
  nest_opts.alpha = lambda;
  [V, obj, msg, pre_L_V] = solve_Nesterov_generic(@obj_V, @L21_loss, @prox_op_L21, V, nest_opts);

  % Fix V, optimize the bases U
  const_V_obj = L21_loss(V);
  nest_opts.alpha = 1;
  nest_opts.init_L = pre_L_U;
  [U, obj, msg, pre_L_U] = solve_Nesterov_generic(@obj_U, [], @prox_op_U, U, nest_opts);
              
  [G1, G2, grads, f] = comp_grad_mv(U*V, opts);    
  obj = w*f + lambda * L21_loss(V);
  fprintf('iter=%d, loss = %g, obj=%g, feval=%d, teval=%g, k=%d, time=%g, msg=%s\n', ...
          i, loss, obj, feval_logistic, time_logistic, k, total_time, msg);
    
  if i > 1 && abs(pre_obj-obj) / min(abs([pre_obj, obj])) < opts.rtol
    msg = 'Stop with small relative change';    
%     disp(msg);
    break;
  else
    pre_obj = obj;
  end

  
  t1 = cputime - t1;  total_time = total_time + t1;
  
  if ~isempty(evaluator)
    perf = evaluator(U);
    fprintf('perf = %g\n', perf);
  end
  
  if make_output
    outputs.obj_list(i) = obj;
    outputs.time_list(i) = total_time;
    outputs.loss_list(i) = loss;
    outputs.perf_list(i) = perf;
  end
end

if make_output
  outputs.obj_list = outputs.obj_list(1:i-1);
  outputs.time_list = outputs.time_list(1:i-1);
  outputs.loss_list = outputs.loss_list(1:i-1);
  outputs.perf_list = outputs.perf_list(1:i-1);
end

sol = U*V;
iter = i;
if i == opts.max_iter
  msg = 'Stop with max iteration';
end  
  
  function [f, g] = obj_V(VV)
%     VV = reshape(x, [k, n]);
    [G1, G2, grads, f] = comp_grad_mv(U*VV, opts);
    
    f = w*f;
        
    G = [G1*w1; G2*w2; grads*w3];
    g = U'*G;
  end

  function [f, g] = obj_U(UU)
%     UU = reshape(x, [m, k]);
    [G1, G2, grads, f] = comp_grad_mv(UU*V, opts);
    
    loss = w*f; 
    f = w*f + lambda*const_V_obj;
       
    G = [G1*w1; G2*w2; grads*w3];

    g = G*V';
  end

  function uu = prox_op_U(uin, opts1)
    
    uu = uin;
    nm = sqrt(sum(uin(1:num_f1,:).^2));    idx = nm > ub1;
    uu(1:num_f1, idx) = uin(1:num_f1, idx) .* repmat(ub1./nm(idx), [num_f1, 1]);
    
    nm = sqrt(sum(uin(num_f1+1:l,:).^2));    idx = nm > ub2;
    uu(1+num_f1:l, idx) = uin(1+num_f1:l, idx) .* repmat(ub2./nm(idx), [num_f2, 1]);

    temp = uin(l+1:end,:);
    nm = abs(temp); idx = nm > 1;
    temp(idx) = temp(idx) ./ nm(idx);
    uu(l+1:end,:) = temp;
  end
end
