#include "mex.h"

// Extract the elements in G*H' (two dense matrices) indexed by jc and ir.
// Usage: b = proj_omega_sub(G, H, jc, ir);
// G: m*r
// H: n*r
// jc, ir: the standard representation of indices of the nonzeros 
//         in the sparse observation matrix
// b: a vector of the elements in G*H' corresponding to the indices jc and ir.
//    It is in Matlab's standard column major order.
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){

  mwSize m = mxGetM(prhs[0]);
  mwSize n = mxGetM(prhs[1]);
  mwSize r = mxGetN(prhs[0]);
  
  double* G = mxGetPr(prhs[0]);
  double* H = mxGetPr(prhs[1]);

  mwIndex* jc = (mwIndex*)mxGetData(prhs[2]);
  mwIndex* ir = (mwIndex*)mxGetData(prhs[3]);
  int nb_samples = mxGetNumberOfElements(prhs[3]);

  plhs[0] = mxCreateDoubleMatrix(nb_samples, 1, mxREAL);
  double* b = mxGetPr(plhs[0]);
  double *p1, *p2;

  mwSize nnz;
  for(mwIndex j = 0; j < n; j++) 
  { 
    nnz = jc[j+1] - jc[j];
    for (mwIndex i = 0; i < nnz; i++)
    {
      p1 = G + *(ir++);
      p2 = H + j;
      *b = 0;
      for(int k = 0; k < r; k++) {
        *b += *p1 * *p2;
        p1 += m; 
        p2 += n;
      }
      b++;
    }    
  }
}
