% Use line search on multiview dataset
%
function [sol, obj, res] = solve_LS_mv(objf, lambda, X0, evalf, opts)

  m1 = opts.prob_par.m1;  
  alpha = opts.prob_par.alpha;
  beta = opts.prob_par.beta;
  UV_opt = 0;
  t1 = 0;
  gvalue = get_global_value();
  
  res = struct('obj_list', [], ...
               'loss_list', [], ...
               'perf_list', [] ...
               );
  
  msg = 'Running';
  perf = struct('l1', 0, 'l2', 0);  
  par = opts.ori_par;
  
  time_list = [];
  
  i = 0;
  init_par = opts.pbm_init_par;
  
  [sol, obj, iter, fval, flag] = pbm1(@obj_ls, init_par.x0, init_par.bl, init_par.bu, opts.pbm_par);
  outputs.msg = msg2str(flag, 'pbm');
  
  if opts.verbose
    fprintf(1, '%s\n', outputs.msg);
  end
  
    % local search objective
    function [f, g] = obj_ls(eta)
      
      % report performance
      i = i + 1;
      t1 = cputime;      
      [UV_opt, f, res] = solve_LOB(@(Z)obj_tr(Z, eta), lambda, X0, [], opts.tr_par);
      f = -f;

      U = UV_opt{1};    V = UV_opt{2}';
      Z1 = U(1:m1,:)*V;   Z2 = U(1+m1:end,:)*V;
      [~, G] = objf.obj([Z1*alpha/eta^0.5; Z2*beta/(1-eta)^0.5]);
      g1 = sum(sum(Z1 .* G(1:m1,:)));
      g2 = sum(sum(Z2 .* G(1+m1:end,:)));
      
      g = 0.5*(alpha*g1*eta^(-1.5) - beta*g2*(1-eta)^(-1.5));

      time_list(end+1) = cputime - t1;
      show_perf(U, V, g, eta);
    end
  
    function [f, G] = obj_tr(Z, eta)
      U = Z{1};    V = Z{2}';
      [f, G] = objf.obj([U(1:m1,:)*(alpha/eta^0.5); U(1+m1:end,:)*(beta/(1-eta)^0.5)]*V);
      G = [G(1:m1,:)*(alpha/eta^0.5); G(1+m1:end,:)*(beta/(1-eta)^0.5)];
    end
  
    function [] = show_perf(U, V, g, eta)
      U = [U(1:m1,:)*(alpha/eta^0.5); U(1+m1:end,:)*(beta/(1-eta)^0.5)];
      ZZ = U*V;
      [loss, ~] = objf.obj(ZZ);
      val = loss + lambda*kappa(U, V, par);
            
      res.loss_list(end+1) = loss;
      res.obj_list(end+1) = val;
      
      if ~isempty(evalf)
        perf = evalf(ZZ);
        res.perf_list = [res.perf_list; perf];
      end
            
      gvalue.col_print('%d: eta = %g, g = %g, obj = %g, perf = (%g, %g), time = %g\n', ...
                        i, eta, g, val, perf.l1, perf.l2, sum(time_list));
      res.time_list = cumsum(time_list);
      res.msg = msg;
      res.iter = i;

      if isfield(opts, 'res_fname')
        save(opts.res_fname, 'res', 'par');
      end

    end
  
end
